rm(list=ls())

library("data.table")
library("BMS")
library("xlsx")
library("RStata")
library("stargazer")
library("readstata13")
library("foreign")
library("ggplot2")
library("reshape2")
library("corrplot")
library("installr")
library("puniform")
library("metafor")
library("dmetar")
library("haven")
library("tidyverse")
library("meta")

#Set directory
setwd("D:/Education/Economics and Finance/IES/Thesis/Meta-analysis/Labor Supply Elasticities/Papers/Frisch (Extensive)/Code")


## Stata data for P-uniform* ----------
data <- read_dta("pcurve_med.dta")

puni_star(yi = data$frischwinmed, vi = data$variancemed, side="right", method="ML",alpha = 0.05, control=list( max.iter=1000,tol=0.001,reps=10000, int=c(0,2), verbose=TRUE))
# alternative
puni_star(tobs = data$tstatcombwinmed , ni = data$nobs_med, alpha = 0.05, side = "right", method = "ML", control=list(stval.tau=0.05, max.iter=1000,tol=0.1,reps=10000, int=c(0,2), verbose=TRUE))



##############################
# Loading data ----------

data_base_comb     = read.dta13("bma_base_comb.dta")
data_ps_comb       = read.dta13("bma_ps_comb.dta")
data_se_comb       = read.dta13("bma_se_comb.dta")


# Columns' names

colnames(data_base_comb)  <- c("ID","Frisch", "Standard error", "Time span", "Monthly", "Quarterly", "Annual", "Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Non-USA", "Indivisible labor", "Micro", "Macro", "Quasi-experimental", "OLS", "Probit", "Non-parametric", "IV", "Quantile", "Publication year", "Top journal","Citations", "googcite","Byproduct")
colnames(data_ps_comb)    <- c("Frisch", "Standard error", "Time span", "Monthly", "Quarterly", "Annual", "Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Non-USA", "Indivisible labor", "Micro", "Macro", "Quasi-experimental", "OLS", "Probit", "Non-parametric", "IV","Quantile","Publication year", "Top journal", "Citations","googcite","Byproduct")
colnames(data_se_comb)    <- c("Frisch", "Time span", "Monthly", "Quarterly", "Annual", "Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Non-USA", "Indivisible labor", "Micro", "Macro", "Quasi-experimental", "OLS", "Probit", "Non-parametric", "IV","Quantile", "Publication year", "Top journal", "Citations","googcite","Byproduct")



keeps <- c("Frisch", "Standard error", "Time span", "Monthly", "Quarterly","Prime age", "Near retirement","Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Indivisible labor", "Macro", "Quasi-experimental", "Probit", "Non-parametric", "IV","Publication year", "Top journal", "Citations","Byproduct")


df_base_comb <- data_base_comb[keeps]
df_ps_comb   <- data_ps_comb[keeps]
df_se_comb   <- data_se_comb[c("Frisch", "Time span", "Monthly", "Quarterly","Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Indivisible labor", "Macro", "Quasi-experimental", "Probit", "Non-parametric", "IV", "Publication year", "Top journal", "Citations","Byproduct")]
df_wse_comb  <- data_base_comb[c("Frisch", "Time span", "Monthly", "Quarterly","Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Indivisible labor", "Macro", "Quasi-experimental", "Probit", "Non-parametric", "IV", "Publication year", "Top journal", "Citations","Byproduct")]

df_wse    <- data_base_comb[c("Frisch", "Time span", "Monthly", "Quarterly","Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Indivisible labor", "Macro", "Quasi-experimental", "Probit", "Non-parametric", "IV", "Publication year", "Top journal", "Citations","Byproduct")]




########## Stem-based method - Furukawa 2019 ----------

source("stem_method.R") #https://github.com/Chishio318/stem-based_method


secomb    <- data_base_comb[1:3]

# Mean values for the sample with combined standard errors 
mcomb<-aggregate(cbind(secomb$Frisch, secomb$`Standard error`), by=list(ID=secomb$ID), mean)
colnames(mcomb) <- c("id","mefr","mese")

# Stem
stem_comb <- stem(mcomb$mefr, mcomb$mese, param)
funnel_comb <- stem_funnel(mcomb$mefr, mcomb$mese, stem_comb$estimates)
View(stem_comb$estimates)


## Correlation matrices ----------

## Correlation matrix for data with re-sampling 

cormatd<-round(cor(subset(df_base_comb, select= -Frisch)),3)
cormatd[upper.tri(cormatd)] <- NA

#Heatmap
pdf("corr_comb2.pdf")
cor1<-corrplot(cormatd,method = "color", tl.col = "black", tl.cex=0.7, cl.cex=0.7,
               col = colorRampPalette(c("darkred","white","midnightblue"))(100), 
               number.digits = 2, number.cex = 0.5, addCoef.col = "black", type= "lower", tl.srt=45)
dev.off()


######### BMA ----------

##Parameters for in-sample estimation

burn_       = 1e6
iter_       = 3e6
gprior      = "UIP"
modelprior  = "uniform"
order_PIP   = F
separator   = ","



## The sample with bootstrapped standard errors -----


## Group PIPs 

bma_group = bms(df_base_comb, burn=1e6,iter=3e6, g="UIP", mprior="dilut", nmodel=10000, mcmc="bd", user.int=FALSE)


se <- c("Standard error")
demographics <- c("Prime age","Near retirement", "Females only", "Males only", "Married", "Single", "Income")
data <- c("Time span", "Monthly", "Quarterly", "Industry", "Macro", "USA")
specifications <- c("Indivisible labor", "Quasi-experimental", "Probit", "Non-parametric", "IV")
publication <- c("Publication year", "Top journal", "Citations","Byproduct")

coefs.smc <- data.table(Variable=row.names(coef(bma_group)),coef(bma_group, exact=T))

coefs.smc[Variable %in% se, Group:="Standard error"]
coefs.smc[Variable %in% demographics, Group:="Demographics"]
coefs.smc[Variable %in% data, Group:="Data characteristics"]
coefs.smc[Variable %in% specifications, Group:="Specifications"]
coefs.smc[Variable %in% publication , Group:="Publication characteristics"]

group.pips <- coefs.smc[,j=.(PIP=1-(prod(1-PIP))), by=c("Group")]

write.csv(group.pips, file="Group PIPs.csv", row.names=F)




## UIP and dilution - With combined standard errors

BMA_dilut = bms(df_base_comb, burn=1e6,iter=3e6, g="UIP", mprior="dilut", nmodel=10000, mcmc="bd", user.int=FALSE)
BMA_dilbase<-coef(BMA_dilut,std.coefs=F, order.by.pip = F, exact=T,include.constant = T)
write.table(BMA_dilbase,"BMA_dilut.csv",sep=separator)
image(BMA_dilut, cex=0.7, xlab="", main="")
stargazer(summary(BMA_dilut), out="bma_dilut.tex",append=TRUE)
plot(BMA_dilut)

#saving plots
pdf("bma_dilut.pdf")
image(BMA_dilut, cex=0.7, xlab="", main="", col=c("deeppink3","darkblue"))
dev.off()

pdf("bma_dilut_pmp.pdf")
plot(BMA_dilut)
dev.off()


## BRIC and Random - With combined standard errors

BMA_base_comb2  = bms(df_base_comb, burn=burn_,iter=iter_, g="BRIC", mprior="random", nmodel=10000, mcmc="bd", user.int=FALSE)
BMA_basec2<-coef(BMA_base_comb2 ,std.coefs=F, order.by.pip = order_PIP, exact=F,include.constant = T)
write.table(BMA_basec2,"BMA_base_comb2.csv",sep=separator)
image(BMA_base_comb2 , cex=0.7, xlab="", main="")
summary(BMA_base_comb2)
plot(BMA_base_comb2)


#saving plots
pdf("bma_comb_bric.pdf")
image(BMA_base_comb2, cex=0.7, xlab="", main="", col=c("deeppink3","darkblue"))
dev.off()

pdf("bma_comb_pmp_bric.pdf")
plot(BMA_base_comb2)
dev.off()


## HQ and Random - With combined standard errors

BMA_base_comb3  = bms(df_base_comb, burn=burn_,iter=iter_, g="HQ", mprior="random", nmodel=10000, mcmc="bd", user.int=FALSE)
BMA_basec3<-coef(BMA_base_comb3 ,std.coefs=F, order.by.pip = order_PIP, exact=F,include.constant = T)
write.table(BMA_basec3,"BMA_base_comb3.csv",sep=separator)
image(BMA_base_comb3 , cex=0.7, xlab="", main="")
summary(BMA_base_comb3)
plot(BMA_base_comb3)


#saving plots
pdf("bma_comb_hq.pdf")
image(BMA_base_comb3, cex=0.7, xlab="", main="", col=c("deeppink3","darkblue"))
dev.off()

pdf("bma_comb_pmp_hq.pdf")
plot(BMA_base_comb3)
dev.off()


## PIP across different priors

pdf("comb321.pdf")
plotComp("UIP and Dilution"=BMA_dilut,"BRIC and Random"=BMA_base_comb2,"HQ and Random"=BMA_base_comb3, add.grid=F, cex.xaxis=0.7)
dev.off()


##############################
### BMA Robustness checks ----------


## The sample with bootstrapped standard errors -----

## Dilution - excluding standard errors

BMA_dilut_wse = bms(df_wse_comb, burn=1e6,iter=3e6, g="UIP", mprior="dilut", nmodel=5000, mcmc="bd", user.int=FALSE)
BMA_dilbasewse<-coef(BMA_dilut_wse,std.coefs=F, order.by.pip = F, exact=T,include.constant = T)
write.table(BMA_dilbasewse,"BMA_dilut_wse.csv",sep=separator)
image(BMA_dilut_wse, cex=0.7, xlab="", main="")
stargazer(summary(BMA_dilut_wse), out="bma_dilut_wse.tex",append=TRUE)
plot(BMA_dilut_wse)

#saving plots
pdf("bma_dilut_wse.pdf")
image(BMA_dilut_wse, cex=0.7, xlab="", main="", col=c("deeppink3","darkblue"))
dev.off()

pdf("bma_dilut_wse_pmp.pdf")
plot(BMA_dilut_wse)
dev.off()



## Dilution - Weighted by the inverse of the number of estimates per study

BMA_dilut_ps = bms(df_ps_comb, burn=1e6,iter=3e6, g="UIP", mprior="dilut", nmodel=5000, mcmc="bd", user.int=FALSE)
BMA_dilbaseps<-coef(BMA_dilut_ps,std.coefs=F, order.by.pip = F, exact=T,include.constant = T)
write.table(BMA_dilbaseps,"BMA_dilut_ps.csv",sep=separator)
image(BMA_dilut_ps, cex=0.7, xlab="", main="")
stargazer(summary(BMA_dilut_ps), out="bma_dilut_ps.tex",append=TRUE)
plot(BMA_dilut_ps)

#saving plots

pdf("bma_dilut_ps.pdf")
image(BMA_dilut_ps, cex=0.7, xlab="", main="", col=c("deeppink3","darkblue"))
dev.off()

pdf("bma_dilut_ps_pmp.pdf")
plot(BMA_dilut_ps)
dev.off()



##############################
######### FMA ----------


library("xtable")
library("LowRankQP")




## The sample with bootstrapped standard errors -----


mydata <- df_base_comb

#colnames(mydata)  <- c("Frisch", "Standard error", "Time span", "Monthly", "Quarterly","Prime age", "Near retirement","Females only", "Males only", "Married", "Single", "Income", "Industry", "USA", "Indivisible labor", "Macro", "Quasi experimental", "Probit", "Non-parametric", "IV", "Publication year", "Top journal", "Citations","Byproduct")
#deleting missing observations
mydata <-na.omit(mydata)
x.data <- mydata[,-1]

#adding constant
const_<-c(1)
x.data <-cbind(const_,x.data)


x <- sapply(1:ncol(x.data),function(i){x.data[,i]/max(x.data[,i])})   
scale.vector <- as.matrix(sapply(1:ncol(x.data),function(i){max(x.data[,i])}))        
Y <- as.matrix(mydata[,1])
output.colnames <- colnames(x.data)
full.fit <- lm(Y~x-1)
beta.full <- as.matrix(coef(full.fit))
M <- k <- ncol(x)
n <- nrow(x)
beta <- matrix(0,k,M)
e <- matrix(0,n,M)
K_vector <- matrix(c(1:M))
var.matrix <- matrix(0,k,M)
bias.sq <- matrix(0,k,M)             


# MMA Estimator using orthogonalization 
for(i in 1:M)
{
  X <- as.matrix(x[,1:i])
  ortho <- eigen(t(X)%*%X)
  Q <- ortho$vectors ; lambda <- ortho$values 
  x.tilda <- X%*%Q%*%(diag(lambda^-0.5,i,i))
  beta.star <- t(x.tilda)%*%Y
  beta.hat <- Q%*%diag(lambda^-0.5,i,i)%*%beta.star
  beta[1:i,i] <- beta.hat
  e[,i] <- Y-x.tilda%*%as.matrix(beta.star)
  bias.sq[,i] <- (beta[,i]-beta.full)^2
  var.matrix.star <- diag(as.numeric(((t(e[,i])%*%e[,i])/(n-i))),i,i)
  var.matrix.hat <- var.matrix.star%*%(Q%*%diag(lambda^-1,i,i)%*%t(Q))
  var.matrix[1:i,i] <- diag(var.matrix.hat)
  var.matrix[,i] <- var.matrix[,i]+ bias.sq[,i]
} # End loop over i

e_k <- e[,M]
sigma_hat <- as.numeric((t(e_k)%*%e_k)/(n-M))
G <- t(e)%*%e
a <- ((sigma_hat)^2)*K_vector
A <- matrix(1,1,M)
b <- matrix(1,1,1)
u <- matrix(1,M,1)
optim <- LowRankQP(Vmat=G,dvec=a,Amat=A,bvec=b,uvec=u,method="LU",verbose=TRUE)
weights <- as.matrix(optim$alpha)
beta.scaled <- beta%*%weights
final.beta <- beta.scaled/scale.vector
std.scaled <- sqrt(var.matrix)%*%weights
final.std <- std.scaled/scale.vector
results.reduced <- as.matrix(cbind(final.beta,final.std))
rownames(results.reduced) <- output.colnames; colnames(results.reduced) <- c("Coefficient", "Sd. Err")
#MMA.fls <- round(results.reduced,4)[-1,]
MMA.fls <- round(results.reduced,4)
list(MMA.fls)


MMA.fls <- data.frame(MMA.fls)
t <- as.data.frame(MMA.fls$Coefficient/MMA.fls$Sd..Err)
MMA.fls$pv <-round( (1-apply(as.data.frame(apply(t,1,abs)), 1, pnorm))*2,3)
MMA.fls$names <- rownames(MMA.fls)
names <- c(colnames(mydata))
names <- c(names,"const_")
MMA.fls <- MMA.fls[match(names, MMA.fls$names),]
MMA.fls$names <- NULL
MMA.fls


library(xtable)
xtable(MMA.fls,digits=c(0,3,3,3))

separator   = ";"
write.table(MMA.fls,"FMA_comb.csv",sep=separator, append = FALSE)





######## End of Code ----------
